@@ -313,3 +313,35 @@ def deploy_remote_core():
         name=project_name, verbose=beanstalk_settings['VERBOSE']))
 
     print separator(' . ')
+
+
+@task
+@beanstalk_role('app')
+def activate(settings_path=None, **settings_patches):
+    """Activate apps in the remote
+    """
+    beanstalk_settings = load_role_settings(additional_settings_path=settings_path, **settings_patches)
+    load_web_servers()
+
+    project_name = beanstalk_settings['PROJECT_NAME']
+
+    @roles('web_servers')
+    def core():
+        run('bsjack server.activate_app:{project_name}'.format(project_name=project_name))
+    execute(core)
+
+
+@task
+@beanstalk_role('app')
+def deactivate(settings_path=None, **settings_patches):
+    """Deactivate apps in the remote
+    """
+    beanstalk_settings = load_role_settings(additional_settings_path=settings_path, **settings_patches)
+    load_web_servers()
+
+    project_name = beanstalk_settings['PROJECT_NAME']
+
+    @roles('web_servers')
+    def core():
+        run('bsjack server.deactivate_app:{project_name}'.format(project_name=project_name))
+    execute(core)
