@@ -2,7 +2,6 @@ import ConfigParser
 import datetime
 import json
 import os
-import socket
 from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.console import confirm
@@ -11,7 +10,7 @@ from ground_soil.filesystem import temporary_directory, render_file, sed
 from beanstalk import IDENTIFIER, BEANSTALK_ROOT_PATH, BEANSTALK_GLOBAL_BASE_PATH
 from beanstalk.decorators import beanstalk_role
 from beanstalk.paths.server import (project_base_path, project_source_path, project_logs_path, project_venv_path,
-                                    project_uwsgi_path, apache_conf_path, app_ports_path)
+                                    project_uwsgi_path, apache_conf_path, project_run_path)
 from beanstalk.tasks.utils import load_role_settings, set_verbose_level, run_hooked_actions
 from beanstalk.console.titles import tool_title, section_title, separator
 
@@ -123,8 +122,6 @@ def setup(settings_path=None, **settings_patches):
     local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'apps'))
     local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'global-logs'))
     local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'confs'))
-    if not os.path.exists(app_ports_path()):
-        local('echo "[]" > %s' % app_ports_path())
     local('chmod o+rx %s' % beanstalk_stack_base)
 
     # Global ini
@@ -205,6 +202,7 @@ def create_app(project_name, settings_path=None, **settings_patches):
     project_source = project_source_path(project_name)
     project_logs = project_logs_path(project_name)
     project_venv = project_venv_path(project_name)
+    project_run = project_run_path(project_name)
 
     if os.path.exists(project_base):
         print 'Nothing to do. App existed!'
@@ -218,7 +216,9 @@ def create_app(project_name, settings_path=None, **settings_patches):
     local('mkdir -p %s' % project_base)
     local('mkdir -p %s' % project_source)
     local('mkdir -p %s' % project_logs)
+    local('mkdir -p %s' % project_run)
     local('setfacl -m u:apache:rwx %s' % project_logs)
+    local('setfacl -m u:apache:rwx %s' % project_run)
 
     # Initialize venv
     if not os.path.exists(project_venv):
@@ -235,56 +235,7 @@ def create_app(project_name, settings_path=None, **settings_patches):
             local('git add .gitignore')
             local('git commit -m "Init commit"')
 
-    # Get uwsgi port
-    with open(app_ports_path(), 'r') as f:
-        ports = json.loads(f.read())
-    if len(ports) == 0:
-        uwsgi_port = 50000
-    else:
-        uwsgi_port = max(ports) + 1
-    # Available ports?
-    try_times = 1
-    port_available = False
-    while try_times <= 10:
-        try:
-            s = socket.socket()
-            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-            s.bind(('localhost', uwsgi_port))
-            s.close()
-            port_available = True
-            break
-        except socket.error:
-            uwsgi_port += 1
-            try_times += 1
-    if not port_available:
-        abort('No available port for uwsgi')
-    else:
-        env.uwsgi_port = uwsgi_port
-        ports.append(uwsgi_port)
-    # Save it
-    with open(app_ports_path(), 'w') as f:
-        f.write(json.dumps(ports, separators=(',', ':')))
-
-    print section_title('Add to apache httpd')
-    with open(apache_conf_path(), 'r') as f:
-        apache_conf = f.read()
-    if not '# %s' % project_name in apache_conf:
-        commands = [
-            '# %s' % project_name,
-            '#' + '-' * 20,
-            '<Location /%s>' % project_name,
-            '    SetHandler uwsgi-handler',
-            '    uWSGISocket 127.0.0.1:%s' % uwsgi_port,
-            '</Location>',
-            '#' + '-' * 20,
-            ' ',
-        ]
-        sed_command = '\\n'.join(commands)
-        local(sed('$ a\\{command}'.format(command=sed_command), apache_conf_path()))
-        local('sudo /usr/sbin/httpd_reload')
-
-    port_file_path = os.path.join(project_base_path(project_name), '%d.port' % uwsgi_port)
-    local('echo "%d" > %s' % (uwsgi_port, port_file_path))
+    execute(activate_app, project_name, settings_path=settings_path, **settings_patches)
 
     # Post-create actions
     run_hooked_actions('POST_CREATE_ACTIONS')
@@ -308,22 +259,9 @@ def delete_app(project_name, settings_path=None, **settings_patches):
     # delete
     if remove:
         print section_title('Remove app @ %s' % project_base)
-        try:
-            port_number = int([name for name in os.listdir(project_base) if name.endswith('.port')][0].split('.')[0])
-            with open(app_ports_path(), 'r') as f:
-                ports = json.loads(f.read())
-            if port_number in ports:
-                ports.remove(port_number)
-            with open(app_ports_path(), 'w') as f:
-                f.write(json.dumps(ports))
-        except (IndexError, ValueError):
-            pass
-
+        execute(deactivate_app, project_name, settings_path=settings_path, **settings_patches)
         local('rm -rf %s' % project_base)
 
-    local(sed('/# {name}/,+7d'.format(name=project_name), apache_conf_path()))
-    local('sudo /usr/sbin/httpd_reload')
-
 
 @task
 @beanstalk_role('server')
@@ -339,6 +277,7 @@ def install_app(project_name, settings_path=None, **settings_patches):
 
     project_source = project_source_path(project_name)
     project_venv = project_venv_path(project_name)
+    project_run = project_run_path(project_name)
 
     print section_title('Install/Update packages in virtualenv')
     with settings(virtual_env(project_venv)):
@@ -350,13 +289,6 @@ def install_app(project_name, settings_path=None, **settings_patches):
     local('python -m compileall -q %s' % project_source)
 
     print section_title('Setup uwsgi')
-    # uwsgi port
-    try:
-        uwsgi_port = int([name for name in os.listdir(project_base_path(project_name))
-                          if name.endswith('.port')][0].split('.')[0])
-    except (IndexError, ValueError):
-        abort('No port number!?')
-        return
     # edit ini
     uwsgi_path = project_uwsgi_path(project_name)
     config = ConfigParser.ConfigParser()
@@ -379,7 +311,7 @@ def install_app(project_name, settings_path=None, **settings_patches):
             config.set('uwsgi', key, value)
     # Options that must be overriden by beanstalk
     must_override_options = [
-        ('socket', '127.0.0.1:%d' % uwsgi_port),
+        ('socket', '%s' % os.path.join(project_run, 'uwsgi.sock')),
         ('chdir', project_source_path(project_name)),
         ('home', project_venv_path(project_name)),
         ('logto', os.path.join(project_logs_path(project_name), 'uwsgi.log')),
@@ -422,6 +354,63 @@ def reload_app(project_name, settings_path=None, **settings_patches):
     run_hooked_actions('POST_RELOAD_ACTIONS')
 
 
+@task
+@beanstalk_role('server')
+def commit_app(project_name, settings_path=None, **settings_patches):
+    """Commit app
+    """
+    settings_patches = eval_kwargs(settings_patches)
+    load_role_settings(additional_settings_path=settings_path, **settings_patches)
+    env.project_name = project_name
+    set_verbose_level()
+
+    run_hooked_actions('PRE_COMMIT_ACTION')
+
+    project_base = project_base_path(project_name)
+    with settings(lcd(project_base), hide('warnings'), warning=True):
+        local('git add -A && git commit -m "state at %s"' % datetime.datetime.now().strftime('%s'))
+
+    run_hooked_actions('POST_COMMIT_ACTION')
+
+
+@task
+@beanstalk_role('server')
+def activate_app(project_name, settings_path=None, **settings_patches):
+    """Add app to apache httpd
+    """
+    print section_title('Add to apache httpd')
+
+    load_role_settings(additional_settings_path=settings_path, **settings_patches)
+    project_run = project_run_path(project_name)
+
+    with open(apache_conf_path(), 'r') as f:
+        apache_conf = f.read()
+    if not '# %s' % project_name in apache_conf:
+        commands = [
+            '# %s' % project_name,
+            '#' + '-' * 20,
+            '<Location /%s>' % project_name,
+            '    SetHandler uwsgi-handler',
+            '    uWSGISocket %s' % os.path.join(project_run, 'uwsgi.sock'),
+            '</Location>',
+            '#' + '-' * 20,
+            ' ',
+        ]
+        sed_command = '\\n'.join(commands)
+        local(sed('$ a\\{command}'.format(command=sed_command), apache_conf_path()))
+        local('sudo /usr/sbin/httpd_reload')
+
+
+@task
+@beanstalk_role('server')
+def deactivate_app(project_name, settings_path=None, **settings_patches):
+    """Remove app from apache httpd
+    """
+    load_role_settings(additional_settings_path=settings_path, **settings_patches)
+    local(sed('/# {name}/,+7d'.format(name=project_name), apache_conf_path()))
+    local('sudo /usr/sbin/httpd_reload')
+
+
 @task
 @beanstalk_role('server')
 def build_venv(project_name, settings_path=None, **settings_patches):
@@ -466,13 +455,6 @@ def app_info(project_name, settings_path=None, **settings_patches):
             print red('No such app')
         return
 
-    try:
-        uwsgi_port = int([name for name in os.listdir(project_base_path(project_name))
-                          if name.endswith('.port')][0].split('.')[0])
-    except (IndexError, ValueError):
-        abort('No port number!?')
-        return
-
     if beanstalk_settings.get('USE_JSON', False):
         result = {
             'path': {
@@ -480,9 +462,7 @@ def app_info(project_name, settings_path=None, **settings_patches):
                 'source': project_source_path(project_name),
                 'logs': project_logs_path(project_name),
                 'venv': project_venv_path(project_name),
-            },
-            'uwsgi': {
-                'port': uwsgi_port,
+                'run': project_run_path(project_name),
             },
         }
         json_string = json.dumps(result, separators=(',', ':'))
@@ -497,8 +477,8 @@ def app_info(project_name, settings_path=None, **settings_patches):
         print 'Source : %s' % project_source_path(project_name)
         print 'Logs   : %s' % project_logs_path(project_name)
         print 'venv   : %s' % project_venv_path(project_name)
+        print 'run    : %s' % project_run_path(project_name)
         print separator('-', 40)
-        print 'uwsgi port : %s' % uwsgi_port
 
 
 @task
@@ -518,25 +498,6 @@ def list_apps(settings_path=None, **settings_patches):
         print app
 
 
-@task
-@beanstalk_role('server')
-def commit_app(project_name, settings_path=None, **settings_patches):
-    """Commit app
-    """
-    settings_patches = eval_kwargs(settings_patches)
-    load_role_settings(additional_settings_path=settings_path, **settings_patches)
-    env.project_name = project_name
-    set_verbose_level()
-
-    run_hooked_actions('PRE_COMMIT_ACTION')
-
-    project_base = project_base_path(project_name)
-    with settings(lcd(project_base), hide('warnings'), warning=True):
-        local('git add -A && git commit -m "state at %s"' % datetime.datetime.now().strftime('%s'))
-
-    run_hooked_actions('POST_COMMIT_ACTION')
-
-
 @task
 @beanstalk_role('server')
 def add_user(tmp_key_path, clean_tmp_key=False):
